@@ -13,7 +13,21 @@ import functools
 import inspect
 import typing
 import uuid
-from typing import Any, Callable, Dict, List, Literal, Optional, Set, Tuple, Type, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generic,
+    List,
+    Literal,
+    NamedTuple,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 from .core import JsonType
 from .exception import JsonKeyError, JsonTypeError, JsonValueError
@@ -36,16 +50,22 @@ from .inspection import (
 from .mapping import python_field_to_json_property
 from .name import python_type_to_str
 
+E = TypeVar("E", bound=enum.Enum)
+T = TypeVar("T")
+R = TypeVar("R")
+K = TypeVar("K")
+V = TypeVar("V")
 
-class Deserializer(abc.ABC):
+
+class Deserializer(abc.ABC, Generic[T]):
     "Parses a JSON value into a Python type."
 
     @abc.abstractmethod
-    def parse(self, data: JsonType) -> Any:
+    def parse(self, data: JsonType) -> T:
         ...
 
 
-class NoneDeserializer(Deserializer):
+class NoneDeserializer(Deserializer[None]):
     "Parses JSON `null` values into Python `None`."
 
     def parse(self, data: JsonType) -> None:
@@ -56,7 +76,7 @@ class NoneDeserializer(Deserializer):
         return None
 
 
-class BoolDeserializer(Deserializer):
+class BoolDeserializer(Deserializer[bool]):
     "Parses JSON `boolean` values into Python `bool` type."
 
     def parse(self, data: JsonType) -> bool:
@@ -67,7 +87,7 @@ class BoolDeserializer(Deserializer):
         return bool(data)
 
 
-class IntDeserializer(Deserializer):
+class IntDeserializer(Deserializer[int]):
     "Parses JSON `number` values into Python `int` type."
 
     def parse(self, data: JsonType) -> int:
@@ -78,7 +98,7 @@ class IntDeserializer(Deserializer):
         return int(data)
 
 
-class FloatDeserializer(Deserializer):
+class FloatDeserializer(Deserializer[float]):
     "Parses JSON `number` values into Python `float` type."
 
     def parse(self, data: JsonType) -> float:
@@ -89,7 +109,7 @@ class FloatDeserializer(Deserializer):
         return float(data)
 
 
-class StringDeserializer(Deserializer):
+class StringDeserializer(Deserializer[str]):
     "Parses JSON `string` values into Python `str` type."
 
     def parse(self, data: JsonType) -> str:
@@ -100,7 +120,7 @@ class StringDeserializer(Deserializer):
         return str(data)
 
 
-class BytesDeserializer(Deserializer):
+class BytesDeserializer(Deserializer[bytes]):
     "Parses JSON `string` values of Base64-encoded strings into Python `bytes` type."
 
     def parse(self, data: JsonType) -> bytes:
@@ -111,7 +131,7 @@ class BytesDeserializer(Deserializer):
         return base64.b64decode(data, validate=True)
 
 
-class DateTimeDeserializer(Deserializer):
+class DateTimeDeserializer(Deserializer[datetime.datetime]):
     "Parses JSON `string` values representing timestamps in ISO 8601 format to Python `datetime` with time zone."
 
     def parse(self, data: JsonType) -> datetime.datetime:
@@ -130,7 +150,7 @@ class DateTimeDeserializer(Deserializer):
         return timestamp
 
 
-class DateDeserializer(Deserializer):
+class DateDeserializer(Deserializer[datetime.date]):
     "Parses JSON `string` values representing dates in ISO 8601 format to Python `date` type."
 
     def parse(self, data: JsonType) -> datetime.date:
@@ -142,7 +162,7 @@ class DateDeserializer(Deserializer):
         return datetime.date.fromisoformat(data)
 
 
-class TimeDeserializer(Deserializer):
+class TimeDeserializer(Deserializer[datetime.time]):
     "Parses JSON `string` values representing time instances in ISO 8601 format to Python `time` type with time zone."
 
     def parse(self, data: JsonType) -> datetime.time:
@@ -154,7 +174,7 @@ class TimeDeserializer(Deserializer):
         return datetime.time.fromisoformat(data)
 
 
-class UUIDDeserializer(Deserializer):
+class UUIDDeserializer(Deserializer[uuid.UUID]):
     "Parses JSON `string` values of UUID strings into Python `uuid.UUID` type."
 
     def parse(self, data: JsonType) -> uuid.UUID:
@@ -165,17 +185,17 @@ class UUIDDeserializer(Deserializer):
         return uuid.UUID(data)
 
 
-class ListDeserializer(Deserializer):
+class ListDeserializer(Deserializer[List[T]]):
     "Recursively de-serializes a JSON array into a Python `list`."
 
-    item_type: type
+    item_type: Type[T]
     item_parser: Deserializer
 
-    def __init__(self, item_type: type) -> None:
+    def __init__(self, item_type: Type[T]) -> None:
         self.item_type = item_type
         self.item_parser = create_deserializer(item_type)
 
-    def parse(self, data: JsonType) -> list:
+    def parse(self, data: JsonType) -> List[T]:
         if not isinstance(data, list):
             type_name = python_type_to_str(self.item_type)
             raise JsonTypeError(
@@ -185,14 +205,14 @@ class ListDeserializer(Deserializer):
         return [self.item_parser.parse(item) for item in data]
 
 
-class DictDeserializer(Deserializer):
+class DictDeserializer(Deserializer[Dict[K, V]]):
     "Recursively de-serializes a JSON object into a Python `dict`."
 
-    key_type: type
-    value_type: type
-    value_parser: Deserializer
+    key_type: Type[K]
+    value_type: Type[V]
+    value_parser: Deserializer[V]
 
-    def __init__(self, key_type: type, value_type: type) -> None:
+    def __init__(self, key_type: Type[K], value_type: Type[V]) -> None:
         self.key_type = key_type
         self.value_type = value_type
         self._check_key_type()
@@ -223,29 +243,29 @@ class DictDeserializer(Deserializer):
         value_type_name = python_type_to_str(self.value_type)
         return f"Dict[{key_type_name}, {value_type_name}]"
 
-    def parse(self, data: JsonType) -> dict:
+    def parse(self, data: JsonType) -> Dict[K, V]:
         if not isinstance(data, dict):
             raise JsonTypeError(
                 f"`type `{self.container_type}` expects JSON `object` data but instead received: {data}"
             )
 
         return dict(
-            (self.key_type(key), self.value_parser.parse(value))
+            (self.key_type(key), self.value_parser.parse(value))  # type: ignore
             for key, value in data.items()
         )
 
 
-class SetDeserializer(Deserializer):
+class SetDeserializer(Deserializer[Set[T]]):
     "Recursively de-serializes a JSON list into a Python `set`."
 
-    member_type: type
+    member_type: Type[T]
     member_parser: Deserializer
 
-    def __init__(self, member_type: type) -> None:
+    def __init__(self, member_type: Type[T]) -> None:
         self.member_type = member_type
         self.member_parser = create_deserializer(member_type)
 
-    def parse(self, data: JsonType) -> set:
+    def parse(self, data: JsonType) -> Set[T]:
         if not isinstance(data, list):
             type_name = python_type_to_str(self.member_type)
             raise JsonTypeError(
@@ -255,13 +275,13 @@ class SetDeserializer(Deserializer):
         return set(self.member_parser.parse(item) for item in data)
 
 
-class TupleDeserializer(Deserializer):
+class TupleDeserializer(Deserializer[Tuple[Any, ...]]):
     "Recursively de-serializes a JSON list into a Python `tuple`."
 
-    item_types: Tuple[type, ...]
-    item_parsers: Tuple[Deserializer, ...]
+    item_types: Tuple[Type[Any], ...]
+    item_parsers: Tuple[Deserializer[Any], ...]
 
-    def __init__(self, item_types: Tuple[type, ...]) -> None:
+    def __init__(self, item_types: Tuple[Type[Any], ...]) -> None:
         self.item_types = item_types
         self.item_parsers = tuple(
             create_deserializer(item_type) for item_type in item_types
@@ -274,7 +294,7 @@ class TupleDeserializer(Deserializer):
         )
         return f"Tuple[{type_names}]"
 
-    def parse(self, data: JsonType) -> tuple:
+    def parse(self, data: JsonType) -> Tuple[Any, ...]:
         if not isinstance(data, list) or len(data) != len(self.item_parsers):
             if not isinstance(data, list):
                 raise JsonTypeError(
@@ -433,40 +453,40 @@ class LiteralDeserializer(Deserializer):
         return value
 
 
-class EnumDeserializer(Deserializer):
+class EnumDeserializer(Deserializer[E]):
     "Returns an enumeration instance based on the enumeration value read from a JSON value."
 
-    enum_type: Type[enum.Enum]
+    enum_type: Type[E]
 
-    def __init__(self, enum_type: type) -> None:
+    def __init__(self, enum_type: Type[E]) -> None:
         self.enum_type = enum_type
 
-    def parse(self, data: JsonType) -> enum.Enum:
+    def parse(self, data: JsonType) -> E:
         return self.enum_type(data)
 
 
-class CustomDeserializer(Deserializer):
+class CustomDeserializer(Deserializer[T]):
     "Uses the `from_json` class method in class to de-serialize the object from JSON."
 
-    converter: Callable[[JsonType], Any]
+    converter: Callable[[JsonType], T]
 
-    def __init__(self, converter: Callable[[JsonType], Any]) -> None:
+    def __init__(self, converter: Callable[[JsonType], T]) -> None:
         self.converter = converter
 
-    def parse(self, data: JsonType) -> Any:
+    def parse(self, data: JsonType) -> T:
         return self.converter(data)
 
 
-class DeferredDeserializer(Deserializer):
+class DeferredDeserializer(Deserializer[T]):
     """
     Dynamically instantiates a deserializer to parse a JSON value.
 
     Required for de-serializing recursively defined types (e.g. tree structures).
     """
 
-    evaluated_type: type
+    evaluated_type: Type[T]
 
-    def __init__(self, evaluated_type: type) -> None:
+    def __init__(self, evaluated_type: Type[T]) -> None:
         self.evaluated_type = evaluated_type
 
     def parse(self, data: JsonType) -> Any:
@@ -474,7 +494,7 @@ class DeferredDeserializer(Deserializer):
         return deserializer.parse(data)
 
 
-class FieldDeserializer(abc.ABC):
+class FieldDeserializer(abc.ABC, Generic[T, R]):
     """
     Deserializes a JSON property into a Python object field.
 
@@ -485,24 +505,24 @@ class FieldDeserializer(abc.ABC):
 
     property_name: str
     field_name: str
-    parser: Deserializer
+    parser: Deserializer[T]
 
     def __init__(
-        self, property_name: str, field_name: str, parser: Deserializer
+        self, property_name: str, field_name: str, parser: Deserializer[T]
     ) -> None:
         self.property_name = property_name
         self.field_name = field_name
         self.parser = parser
 
     @abc.abstractmethod
-    def parse_field(self, data: Dict[str, JsonType]) -> Any:
+    def parse_field(self, data: Dict[str, JsonType]) -> R:
         ...
 
 
-class RequiredFieldDeserializer(FieldDeserializer):
+class RequiredFieldDeserializer(FieldDeserializer[T, T]):
     "Deserializes a JSON property into a mandatory Python object field."
 
-    def parse_field(self, data: Dict[str, JsonType]) -> Any:
+    def parse_field(self, data: Dict[str, JsonType]) -> T:
         if self.property_name not in data:
             raise JsonKeyError(
                 f"missing required property `{self.property_name}` from JSON object: {data}"
@@ -511,10 +531,10 @@ class RequiredFieldDeserializer(FieldDeserializer):
         return self.parser.parse(data[self.property_name])
 
 
-class OptionalFieldDeserializer(FieldDeserializer):
+class OptionalFieldDeserializer(FieldDeserializer[T, Optional[T]]):
     "Deserializes a JSON property into an optional Python object field with a default value of `None`."
 
-    def parse_field(self, data: Dict[str, JsonType]) -> Any:
+    def parse_field(self, data: Dict[str, JsonType]) -> Optional[T]:
         value = data.get(self.property_name)
         if value is not None:
             return self.parser.parse(value)
@@ -522,22 +542,22 @@ class OptionalFieldDeserializer(FieldDeserializer):
             return None
 
 
-class DefaultFieldDeserializer(FieldDeserializer):
-    "Deserializes a JSON property into an optional Python object field with an explicit default value."
+class DefaultFieldDeserializer(FieldDeserializer[T, T]):
+    "Deserializes a JSON property into a Python object field with an explicit default value."
 
-    default_value: Optional[Any] = None
+    default_value: T
 
     def __init__(
         self,
         property_name: str,
         field_name: str,
         parser: Deserializer,
-        default_value: Optional[Any],
+        default_value: T,
     ) -> None:
         super().__init__(property_name, field_name, parser)
         self.default_value = default_value
 
-    def parse_field(self, data: Dict[str, JsonType]) -> Any:
+    def parse_field(self, data: Dict[str, JsonType]) -> T:
         value = data.get(self.property_name)
         if value is not None:
             return self.parser.parse(value)
@@ -545,22 +565,22 @@ class DefaultFieldDeserializer(FieldDeserializer):
             return self.default_value
 
 
-class DefaultFactoryFieldDeserializer(FieldDeserializer):
+class DefaultFactoryFieldDeserializer(FieldDeserializer[T, T]):
     "Deserializes a JSON property into an optional Python object field with an explicit default value factory."
 
-    default_factory: Callable[[], Any]
+    default_factory: Callable[[], T]
 
     def __init__(
         self,
         property_name: str,
         field_name: str,
-        parser: Deserializer,
-        default_factory: Callable[[], Any],
+        parser: Deserializer[T],
+        default_factory: Callable[[], T],
     ) -> None:
         super().__init__(property_name, field_name, parser)
         self.default_factory = default_factory
 
-    def parse_field(self, data: Dict[str, JsonType]) -> Any:
+    def parse_field(self, data: Dict[str, JsonType]) -> T:
         value = data.get(self.property_name)
         if value is not None:
             return self.parser.parse(value)
@@ -568,16 +588,16 @@ class DefaultFactoryFieldDeserializer(FieldDeserializer):
             return self.default_factory()
 
 
-class ClassDeserializer(Deserializer):
+class ClassDeserializer(Deserializer[T]):
     "Base class for de-serializing class-like types such as data classes, named tuples and regular classes."
 
     class_type: type
     property_parsers: List[FieldDeserializer]
 
-    def __init__(self, class_type: type) -> None:
+    def __init__(self, class_type: Type[T]) -> None:
         self.class_type = class_type
 
-    def parse(self, data: JsonType) -> object:
+    def parse(self, data: JsonType) -> T:
         if not isinstance(data, dict):
             type_name = python_type_to_str(self.class_type)
             raise JsonTypeError(
@@ -604,19 +624,19 @@ class ClassDeserializer(Deserializer):
 
         return self.create(**field_values)
 
-    def create(self, **field_values) -> object:
+    def create(self, **field_values) -> T:
         "Instantiates an object with a collection of property values."
 
-        obj: object = create_object(self.class_type)
+        obj: T = create_object(self.class_type)
         for field_name, field_value in field_values.items():
             setattr(obj, field_name, field_value)
         return obj
 
 
-class NamedTupleDeserializer(ClassDeserializer):
+class NamedTupleDeserializer(ClassDeserializer[NamedTuple]):
     "De-serializes a named tuple from a JSON `object`."
 
-    def __init__(self, class_type: type) -> None:
+    def __init__(self, class_type: Type[NamedTuple]) -> None:
         super().__init__(class_type)
 
         self.property_parsers = [
@@ -626,14 +646,14 @@ class NamedTupleDeserializer(ClassDeserializer):
             for field_name, field_type in get_resolved_hints(class_type).items()
         ]
 
-    def create(self, **field_values) -> object:
+    def create(self, **field_values) -> NamedTuple:
         return self.class_type(**field_values)
 
 
-class DataclassDeserializer(ClassDeserializer):
+class DataclassDeserializer(ClassDeserializer[T]):
     "De-serializes a data class from a JSON `object`."
 
-    def __init__(self, class_type: type) -> None:
+    def __init__(self, class_type: Type[T]) -> None:
         super().__init__(class_type)
 
         self.property_parsers = []
@@ -647,7 +667,7 @@ class DataclassDeserializer(ClassDeserializer):
             has_default_factory = field.default_factory is not dataclasses.MISSING
 
             if is_optional:
-                required_type: type = unwrap_optional_type(field_type)
+                required_type: Type[T] = unwrap_optional_type(field_type)
             else:
                 required_type = field_type
 
@@ -674,10 +694,10 @@ class DataclassDeserializer(ClassDeserializer):
             self.property_parsers.append(field_parser)
 
 
-class TypedClassDeserializer(ClassDeserializer):
+class TypedClassDeserializer(ClassDeserializer[T]):
     "De-serializes a class with type annotations from a JSON `object` by iterating over class properties."
 
-    def __init__(self, class_type: type) -> None:
+    def __init__(self, class_type: Type[T]) -> None:
         super().__init__(class_type)
 
         self.property_parsers = []
@@ -687,7 +707,7 @@ class TypedClassDeserializer(ClassDeserializer):
             is_optional = is_type_optional(field_type)
 
             if is_optional:
-                required_type: type = unwrap_optional_type(field_type)
+                required_type: Type[T] = unwrap_optional_type(field_type)
             else:
                 required_type = field_type
 
@@ -705,7 +725,7 @@ class TypedClassDeserializer(ClassDeserializer):
             self.property_parsers.append(field_parser)
 
 
-def create_deserializer(typ: type) -> Deserializer:
+def create_deserializer(typ: Type[T]) -> Deserializer[T]:
     """
     Creates a de-serializer engine to parse an object obtained from a JSON string.
 
@@ -725,20 +745,26 @@ def create_deserializer(typ: type) -> Deserializer:
     """
 
     if isinstance(typ, type):
-        return _fetch_deserializer(typ)
+        return _fetch_deserializer(typ)  # type: ignore
     else:
         # special forms are not always hashable
         return _create_deserializer(typ)
 
 
 @functools.lru_cache(maxsize=None)
-def _fetch_deserializer(typ: type) -> Deserializer:
+def _fetch_deserializer(typ: Type[T]) -> Deserializer[T]:
     "Creates or re-uses a de-serializer engine to parse an object obtained from a JSON string."
 
     return _create_deserializer(typ)
 
 
-def _create_deserializer(typ: type) -> Deserializer:
+def _create_deserializer(typ: Type[T]) -> Deserializer[T]:
+    "Creates a de-serializer engine to parse an object obtained from a JSON string."
+
+    return _create_deserializer_unsafe(typ)
+
+
+def _create_deserializer_unsafe(typ: Type[T]) -> Deserializer:
     "Creates a de-serializer engine to parse an object obtained from a JSON string."
 
     # check for well-known types
@@ -804,8 +830,7 @@ def _create_deserializer(typ: type) -> Deserializer:
 
     if isinstance(typ, typing.ForwardRef):
         fwd: typing.ForwardRef = typ
-        identifier = fwd.__forward_arg__
-        evaluated_type = eval(identifier)
+        evaluated_type = eval(fwd.__forward_code__)
         return DeferredDeserializer(evaluated_type)
 
     if not inspect.isclass(typ):
