@@ -63,7 +63,7 @@ def is_named_tuple_instance(obj: object) -> TypeGuard[NamedTuple]:
     return is_named_tuple_type(type(obj))
 
 
-def is_named_tuple_type(typ: type) -> TypeGuard[Type[NamedTuple]]:
+def is_named_tuple_type(typ: Type[T]) -> TypeGuard[Type[NamedTuple]]:
     """
     True if the argument corresponds to a named tuple type.
 
@@ -92,8 +92,11 @@ def is_type_enum(typ: type) -> TypeGuard[Type[enum.Enum]]:
 
     typ = unwrap_annotated_type(typ)
 
-    # use an explicit isinstance(..., type) check to filter out special forms like generics
-    return isinstance(typ, type) and issubclass(typ, enum.Enum)
+    if sys.version_info >= (3, 11):
+        return isinstance(typ, enum.EnumType)
+    else:
+        # use an explicit isinstance(..., type) check to filter out special forms like generics
+        return isinstance(typ, type) and issubclass(typ, enum.Enum)
 
 
 def enum_value_types(enum_type: Type[enum.Enum]) -> List[type]:
@@ -310,7 +313,7 @@ def get_annotation(data_type: type, annotation_type: Type[T]) -> Optional[T]:
     return None
 
 
-def unwrap_annotated_type(typ: type) -> type:
+def unwrap_annotated_type(typ: Type[T]) -> Type[T]:
     "Extracts the wrapped type from an annotated type (e.g. returns `T` for `Annotated[T, ...]`)."
 
     if is_type_annotated(typ):
@@ -366,14 +369,10 @@ def get_resolved_hints(typ: type) -> Dict[str, type]:
 def get_class_properties(typ: type) -> Iterable[Tuple[str, type]]:
     "Returns all properties of a class."
 
-    resolved_hints = get_resolved_hints(typ)
-
     if is_dataclass_type(typ):
-        return (
-            (field.name, resolved_hints[field.name])
-            for field in dataclasses.fields(typ)
-        )
+        return ((field.name, field.type) for field in dataclasses.fields(typ))
     else:
+        resolved_hints = get_resolved_hints(typ)
         return resolved_hints.items()
 
 
@@ -436,7 +435,10 @@ def is_reserved_property(name: str) -> bool:
 
 
 def create_object(typ: Type[T]) -> T:
+    "Creates an instance of a type."
+
     if issubclass(typ, Exception):
+        # exception types need special treatment
         e = typ.__new__(typ)
         return typing.cast(T, e)
     else:
