@@ -5,12 +5,7 @@ Type-safe data interchange for Python data classes.
 """
 
 import json
-from typing import (
-    Any,
-    TextIO,
-    Type,
-    TypeVar,
-)
+from typing import Any, TextIO, Type, TypeVar
 
 from .core import JsonType
 from .deserializer import create_deserializer
